<?php

namespace Litepie\Log\Actions;

use Illuminate\Support\Carbon;
use Illuminate\Support\Str;
use Litepie\Actions\Concerns\AsAction;
use Litepie\Actions\Traits\LogsActions;
use Litepie\Notification\Traits\SendNotification;
use Litepie\Log\Models\Activity;


class ActivityAction
{
    use AsAction;

    protected $model;
    protected $namespace = 'litepie.log.activity';
    protected $eventClass = \Litepie\Log\Events\ActivityAction::class;
    protected $action;
    protected $function;
    protected $request;

    public function handle(string $action, Activity $activity, array $request = [])
    {
        $this->action = $action;
        $this->request = $request;
        $this->model = $activity;
        $this->function = Str::camel($action);
        $this->executeAction();
        return $this->model;

    }


    public function store(Activity $activity, array $request)
    {
        $attributes = $request;
        $attributes['user_id'] = user_id();
        $attributes['user_type'] = user_type();
        $activity = $activity->create($attributes);
        return $activity;
    }

    public function update(Activity $activity, array $request)
    {
        $attributes = $request;
        $activity->update($attributes);
        return $activity;
    }

    public function destroy(Activity $activity, array $request)
    {
        $activity->delete();
        return $activity;
    }

    public function copy(Activity $activity, array $request)
    {
        $count = $request['count'] ?: 1;

        if ($count == 1) {
            $activity = $activity->replicate();
            $activity->created_at = Carbon::now();
            $activity->save();
            return $activity;
        }

        for ($i = 1; $i <= $count; $i++) {
            $new = $activity->replicate();
            $new->created_at = Carbon::now();
            $new->save();
        }

        return $activity;
    }


}
