<?php

namespace Litepie\Log\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Litepie\Log\Events\ActionAction as ActionActionEvent;
use Litepie\Log\Notifications\ActionAction as ActionActionNotification;
use Litepie\Actions\Concerns\AsAction;

class ActionAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the ActionActionEvent.
     *
     * @param   ActionActionEvent  $event
     * @return mixed
     */
    public function handle(ActionActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $action.
     *
     * @param   ActionActionEvent  $event
     * @return void
     */
    public function create(ActionActionEvent $event)
    {
        $client = $event->$action->client;
        Notification::send($client, new ActionActionNotification($event));
    }

    /**
     * Handle the ActionActionEvent as a listener.
     *
     * @param   ActionActionEvent  $event
     * @return mixed
     */
    public function asListener(ActionActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   ActionActionEvent  $event
     * @return bool
     */
    private function isAllowed(ActionActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
