<?php

namespace Litepie\Log\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Litepie\Log\Events\ActionWorkflow as ActionWorkflowEvent;
use Litepie\Log\Notifications\ActionWorkflow as ActionWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class ActionWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(ActionWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $action is submitted.
     *
     * @param ActionWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(ActionWorkflowEvent $event)
    {
        $client = $event->$action->client;
        Notification::send($client, new ActionWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $action is published.
     *
     * @param ActionWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(ActionWorkflowEvent $event)
    {
        $client = $event->$action->client;
        Notification::send($client, new ActionWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $action is approved.
     *
     * @param ActionWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(ActionWorkflowEvent $event)
    {
        $client = $event->$action->client;
        Notification::send($client, new ActionWorkflowNotification($event));
    }

    /**
     * Handles the $action workflow event as a listener.
     *
     * @param ActionWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(ActionWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
